use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AALLAN
Alasdair
Bell
DDD
DROLSKY
DROLSKY's
DateTime
Dave
Doug
Format
Hoblitt
ISO8601
Iain
JOHNL
Joshua
Katana
Klausner
Leffler
Liam
MMDD
McCauley
PDX
PayPal
Rolsky
Rolsky's
Thomas
Truskett
Types
Widdowson
Www
WwwD
YWww
YWwwD
YY
YYDDD
YYMM
YYMMDD
YYWww
YYWwwD
YYYY
YYYYDDD
YYYYDDDThhmm
YYYYDDDThhmmZ
YYYYMMDD
YYYYMMDDThhmm
YYYYMMDDThhmmZ
YYYYMMDDThhmmss
YYYYMMDDThhmmssZ
YYYYWww
YYYYWwwD
autarch
domm
doug
draxil
drolsky
hh
hhZ
hhmm
hhmmZ
hhmmss
hhmmssZ
joe
josh
lbw
lib
mmss
ss
ssZ
